@@ -1,3 +1,4 @@
+import logging
 from xmlclient import _tPartnerNS, _tSObjectNS
 from xmlclient import Client as BaseClient
 from marshall import marshall
@@ -12,6 +13,8 @@ _tSchemaNS = Namespace('http://www.w3.org/2001/XMLSchema')
 DEFAULT_FIELD_TYPE = "string"
 querytyperegx = re.compile('(?:from|FROM) (\S+)')
 
+_logger = logging.getLogger("pyforce.{0}".format(__name__))
+
 class QueryRecord(dict):
 
     def __getattr__(self, n):
@@ -67,7 +70,8 @@ class Client(BaseClient):
     def __init__(self, serverUrl=None, cacheTypeDescriptions=False):
         BaseClient.__init__(self, serverUrl=serverUrl)
         self.cacheTypeDescriptions = cacheTypeDescriptions
-        self.typeDescs = {}
+        if self.cacheTypeDescriptions:
+            self.typeDescs = {}
 
     def login(self, username, passwd):
         res = BaseClient.login(self, username, passwd)
@@ -197,6 +201,28 @@ class Client(BaseClient):
                 d['errors'] = list()
         return data
 
+    def convert_leads(self, lead_converts):
+        preparedLeadConverts = _prepareSObjects(lead_converts)
+        del preparedLeadConverts['fieldsToNull']
+        res = BaseClient.convertLeads(self, preparedLeadConverts)
+        if type(res) not in (TupleType, ListType):
+            res = [res]
+        data = list()
+        for resu in res:
+            d = dict()
+            data.append(d)
+            d['success'] = success = _bool(resu[_tPartnerNS.success])
+            if not success:
+                d['errors'] = [_extractError(e)
+                               for e in resu[_tPartnerNS.errors:]]
+            else:
+                d['errors'] = list()
+                d['account_id'] = str(resu[_tPartnerNS.accountId])
+                d['contact_id'] = str(resu[_tPartnerNS.contactId])
+                d['lead_id'] = str(resu[_tPartnerNS.leadId])
+                d['opportunity_id'] = str(resu[_tPartnerNS.opportunityId])
+        return data
+
     def retrieve(self, fields, sObjectType, ids):
         resultSet = BaseClient.retrieve(self, fields, sObjectType, ids)
         type_data = self.describeSObjects(sObjectType)[0]
@@ -235,6 +261,9 @@ class Client(BaseClient):
 
     def queryTypesDescriptions(self, types):
         """
+        Given a list of types, construct a dictionary such that
+        each key is a type, and each value is the corresponding sObject
+        for that type.
         """
         types = list(types)
         if types:
@@ -243,16 +272,19 @@ class Client(BaseClient):
             types_descs = []
         return dict(map(lambda t, d:(t, d), types, types_descs))
 
-    def _extractRecord(self, r):
+    def _extractRecord(self, r, typeDescs):
         record = QueryRecord()
         if r:
-            type_data = self.typeDescs[str(r[_tSObjectNS.type])]
+            row_type = str(r[_tSObjectNS.type])
+            _logger.debug("row type: {0}".format(row_type))
+            type_data = typeDescs[row_type]
+            _logger.debug("type data: {0}".format(type_data))
             for field in r:
                fname = str(field._name[1]) 
                if isObject(field):
-                   record[fname] = self._extractRecord(r[field._name:][0])
+                   record[fname] = self._extractRecord(r[field._name:][0], typeDescs)
                elif isQueryResult(field):
-                   record[fname] = QueryRecordSet(records=[self._extractRecord(rec) for rec in field[_tPartnerNS.records:]],
+                   record[fname] = QueryRecordSet(records=[self._extractRecord(rec, typeDescs) for rec in field[_tPartnerNS.records:]],
                                                   done=field[_tPartnerNS.done],
                                                   size=int(str(field[_tPartnerNS.size]))
                                                  )
@@ -260,10 +292,12 @@ class Client(BaseClient):
                    record[fname] = type_data.marshall(fname, r)
         return record
 
-    def flushTypeDescriptionsCache(self):
-        self.typeDescs = {}
-
     def query(self, *args, **kw):
+        if self.cacheTypeDescriptions:
+            typeDescs = self.typeDescs
+        else:
+            typeDescs = {}
+
         if len(args) == 1: # full query string
             queryString = args[0]
         elif len(args) == 2: # BBB: fields, sObjectType
@@ -279,43 +313,48 @@ class Client(BaseClient):
         res = BaseClient.query(self, queryString)
         # calculate the union of the sets of record types from each record
         types = reduce(lambda a,b: a|b, [getRecordTypes(r) for r in res[_tPartnerNS.records:]], set())
-        if not self.cacheTypeDescriptions:
-            self.flushTypeDescriptionsCache()
-        new_types = types - set(self.typeDescs.keys())
+        new_types = types - set(typeDescs.keys())
         if new_types:
-            self.typeDescs.update(self.queryTypesDescriptions(new_types))
-        data = QueryRecordSet(records=[self._extractRecord(r) for r in res[_tPartnerNS.records:]],
+            typeDescs.update(self.queryTypesDescriptions(new_types))
+        data = QueryRecordSet(records=[self._extractRecord(r, typeDescs) for r in res[_tPartnerNS.records:]],
                               done=_bool(res[_tPartnerNS.done]),
                               size=int(str(res[_tPartnerNS.size])),
                               queryLocator = str(res[_tPartnerNS.queryLocator]))
         return data
 
     def queryMore(self, queryLocator):
+        if self.cacheTypeDescriptions:
+            typeDescs = self.typeDescs
+        else:
+            typeDescs = {}
+
         locator = queryLocator
         res = BaseClient.queryMore(self, locator)
         # calculate the union of the sets of record types from each record
         types = reduce(lambda a,b: a|b, [getRecordTypes(r) for r in res[_tPartnerNS.records:]], set())
-        new_types = types - set(self.typeDescs.keys())
+        new_types = types - set(typeDescs.keys())
         if new_types:
-            self.typeDescs.update(self.queryTypesDescriptions(new_types))
-        data = QueryRecordSet(records=[self._extractRecord(r) for r in res[_tPartnerNS.records:]],
+            typeDescs.update(self.queryTypesDescriptions(new_types))
+        data = QueryRecordSet(records=[self._extractRecord(r, typeDescs) for r in res[_tPartnerNS.records:]],
                               done=_bool(res[_tPartnerNS.done]),
                               size=int(str(res[_tPartnerNS.size])),
                               queryLocator = str(res[_tPartnerNS.queryLocator]))
         return data
 
     def search(self, sosl):
+        if self.cacheTypeDescriptions:
+            typeDescs = self.typeDescs
+        else:
+            typeDescs = {}
         res = BaseClient.search(self, sosl)
 
-        if not self.cacheTypeDescriptions:
-            self.flushTypeDescriptionsCache()
         # calculate the union of the sets of record types from each record
         if len(res):
             types = reduce(lambda a,b: a|b, [getRecordTypes(r) for r in res[_tPartnerNS.searchRecords]], set())
-            new_types = types - set(self.typeDescs.keys())
+            new_types = types - set(typeDescs.keys())
             if new_types:
-                self.typeDescs.update(self.queryTypesDescriptions(new_types))
-            return [self._extractRecord(r) for r in res[_tPartnerNS.searchRecords]]
+                typeDescs.update(self.queryTypesDescriptions(new_types))
+            return [self._extractRecord(r, typeDescs) for r in res[_tPartnerNS.searchRecords]]
         else:
             return []
 
@@ -407,37 +446,37 @@ class Field(object):
     def marshall(self, xml):
         return marshall(self.type, self.name, xml)
 
+def _doPrep(field_dict):
+    """
+    _doPrep is makes changes in-place.
+    Do some prep work converting python types into formats that
+    Salesforce will accept.
+    This includes converting lists of strings to "apple;orange;pear".
+    Dicts will be converted to embedded objects
+    None or empty list values will be Null-ed
+    """
+    fieldsToNull = []
+    for k,v in field_dict.items():
+        if v is None:
+            fieldsToNull.append(k)
+            field_dict[k] = []
+        if hasattr(v,'__iter__'):
+            if len(v) == 0:
+                fieldsToNull.append(k)
+            elif isinstance(v, dict):
+                innerCopy = copy.deepcopy(v)
+                _doPrep(innerCopy)
+                field_dict[k] = innerCopy
+            else:
+                field_dict[k] = ";".join(v)
+    if 'fieldsToNull' in field_dict:
+        raise ValueError, "fieldsToNull should be populated by the client, not the caller."
+    field_dict['fieldsToNull'] = fieldsToNull
 
 # sObjects can be 1 or a list. If values are python lists or tuples, we
 # convert these to strings:
 # ['one','two','three'] becomes 'one;two;three'
 def _prepareSObjects(sObjects):
-     def _doPrep(field_dict):
-         """
-         _doPrep is makes changes in-place.
-         Do some prep work converting python types into formats that
-         Salesforce will accept.
-         This includes converting lists of strings to "apple;orange;pear".
-         Dicts will be converted to embedded objects
-         None or empty list values will be Null-ed
-         """
-         fieldsToNull = []
-         for k,v in field_dict.items():
-             if v is None:
-                 fieldsToNull.append(k)
-                 field_dict[k] = []
-             if hasattr(v,'__iter__'):
-                 if len(v) == 0:
-                     fieldsToNull.append(k)
-                 elif isinstance(v, dict):
-                     innerCopy = copy.deepcopy(v)
-                     _doPrep(innerCopy)
-                     field_dict[k] = innerCopy
-                 else:
-                     field_dict[k] = ";".join(v)
-         if 'fieldsToNull' in field_dict:
-             raise ValueError, "fieldsToNull should be populated by the client, not the caller."
-         field_dict['fieldsToNull'] = fieldsToNull
 
      sObjectsCopy = copy.deepcopy(sObjects)
      if isinstance(sObjectsCopy,dict):
